<?php 

/**
*Image store - core
*
*@package Image Store
*@author Hafid Trujillo
*@copyright 20010-2012
*@since 3.0.0
*/

class ImStore{
	
	/**
	*Constant variables
	*
	*@param $domain plugin Gallery IDentifier
	*Make sure that new language( .mo ) files have 'ims-' as base name
	*/
	public $domain	= 'ims';
	public $version	= '3.0.5';

	/**
	*Public variables
	*/
	public $dformat = '';
	public $sync    = false;
	public $perma   = false;
	public $blog_id = false;
	public $color	= array( );
	public $opts	= array( );
	public $pages	= array( );
	public $promo_types = array( );
	public $rules_property = array( );
	public $optionkey = 'ims_front_options';
	
	/**
	*Constructor
	*
	*@return void
	*@since 0.5.0 
	*/
	function ImStore( ){
		global $wp_version;
		
		$this->define_constant( );
		$this->wp_version = $wp_version;

		$this->content_url = rtrim( WP_CONTENT_URL, '/' );
		$this->content_dir = rtrim( WP_CONTENT_DIR, '/' );

		if( is_multisite( ) && isset( $GLOBALS['blog_id'] ) ){
			$this->blog_id = (int) $GLOBALS['blog_id'];
			$this->sync = get_site_option( 'ims_sync_settings' );
			$this->content_url  = get_site_url( 1 ) . "/wp-content";
		}
		
		if( empty( $this->opts ) && $this->sync == true )
			switch_to_blog( 1 );
		
		$this->opts = get_option( $this->optionkey );
		if( !isset(  $this->opts['attchlink']	) )
			$this->opts['attchlink'] = false;
			
		if( is_multisite( ) ) restore_current_blog( );
		
		add_filter( 'posts_orderby', array( &$this, 'posts_orderby' ), 10,3 );
		add_filter( 'post_type_link', array( &$this, 'gallery_permalink' ), 10,3 );
		
		add_action( 'init', array( &$this, 'int_actions' ),0 );
		add_action( 'wp_loaded', array( &$this, 'flush_rules') );
		add_action( 'wp_logout', array( &$this, 'logout_ims_user' ),10 );
		add_action( 'imstore_expire', array( &$this, 'expire_galleries' ) );
		add_action( 'set_current_user', array( &$this, 'set_user_caps' ),10 );
		add_action( 'generate_rewrite_rules', array( &$this, 'add_rewrite_rules' ),10,1 );
	}
	
	/**
	*Initial actions
	*
	*@return void
	*@since 0.5.0 
	*/
	function int_actions( ){
		
		$this->load_text_domain( );
		$this->register_post_types( );
		
		if( isset( $this->opts['imswidget'] ) && $this->opts['imswidget'] == true ) 
			include_once( apply_filters( 'ims_widget_path', IMSTORE_ABSPATH . '/_inc/widget.php' ) );
		
		//speed up wordpress load
		if( defined( 'DOING_AJAX' ) || defined( 'DOING_AUTOSAVE' || defined( 'SHORTINIT')) ) 
			return;
		
		if( empty( $this->pages ) )
			$this->load_pages( );
		$this->load_color_opts( );
		
		$this->loc 		= $this->opts['clocal'];
		$this->sym 		= $this->opts['symbol']; 
		
		$this->dformat 	= get_option( 'date_format' );
		$this->perma 	= get_option( 'permalink_structure' );
		
		$this->cformat	= array( '',"$this->sym%s", "$this->sym %s", "%s$this->sym", "%s $this->sym" );
		$this->units 		= apply_filters( 'ims_units', array( 
			'in' => __( 'in', $this->domain ), 'cm' => __( 'cm', $this->domain ), 'px' => __( 'px', $this->domain ) 
		) );
		
		$this->promo_types = apply_filters( 'ims_promo_types', array(
			'1' => __( 'Percent', $this->domain ),
			'2'	=> __( 'Amount', $this->domain ), 
			'3' => __( 'Free Shipping', $this->domain ),
		) );
		
		$this->rules_property = apply_filters( 'ims_rules_property', array(
			'items'	=> __( 'Item quantity', $this->domain ),
			'total'		=> __( 'Total amount', $this->domain ),
			'subtotal'=> __( 'Subtotal amount', $this->domain ),
		) );
		
		$this->rules_logic = apply_filters( 'ims_rules_logic', array(
			'equal'	=> __( 'Is equal to', $this->domain ),
			'more'	=> __( 'Is greater than', $this->domain ),
			'less'		=> __( 'Is less than', $this->domain ),
		) );
		
	}
	
	/**
	*Flash rules
	*
	*@return void
	*@since 3.0.0
	*/
	function flush_rules(){
		$rules = get_option( 'rewrite_rules' );
		$galleries = preg_match( '/[^\\p{Common}\\p{Latin}]/u', __( 'galleries', $this->domain ) ) 
		? 'galleries' : __( 'galleries', $this->domain );
		
		if ( ! isset( $rules[$galleries . "/([^/]+)/feed/(imstore)/?$"] ) ) {
			global $wp_rewrite;
			$wp_rewrite->flush_rules();
		}
	}
	
	/**
	*Allow post to be sorted by excerpt
	*
	*@param string $orderby
	*@param obj $query
	*@return string
	*@since 3.0.0
	*/
	function posts_orderby( $orderby, $query ){
		if( empty($query->query_vars['orderby'] ) 
		|| empty( $query->query['orderby'] ) 
		|| $query->query['orderby'] != 'excerpt' )
			return $orderby;
			
		global $wpdb;
		return $wpdb->posts . ".post_excerpt";
	}

	/**
	*Add support to archives permalink
	*
	*@param string $permalink
	*@param obj $post
	*@param string $leavename
	*@return string
	*@since 3.0.0
	*/
	function gallery_permalink( $permalink, $post ){
		if( $post->post_type != 'ims_gallery' ) 
			return $permalink;
		return trim( str_replace( '%imspage%', '', $permalink ) , '/' ) ;
	}
	
	/**
	*logout user 
	*
	*@return void
	*@since 0.5.0 
	*/
	function logout_ims_user( ){
		setcookie( 'ims_galid_' . COOKIEHASH, false, ( time()-315360000 ), COOKIEPATH, COOKIE_DOMAIN );
		setcookie( 'wp-postpass_' . COOKIEHASH, false, ( time()-315360000 ), COOKIEPATH, COOKIE_DOMAIN );
	}
	
	/**
	*Set galleries to expired
	*and delete unprocess orders
	*
	*@return void
	*@since 0.5.0 
	*/
	function expire_galleries( ){
		global $wpdb;
		
		do_action( 'ims_before_cron' );
		$time = date( 'Y-m-d', current_time( 'timestamp' ) );
		
		$wpdb->query(
			"UPDATE $wpdb->posts SET post_status = 'expire'  WHERE post_expire <= '$time'
			AND post_expire != '0000-00-00 00:00:00' AND post_type = 'ims_gallery'");
		
		$wpdb->query( "DELETE p,pm FROM $wpdb->posts p  LEFT JOIN $wpdb->postmeta pm ON(p.ID = pm.post_id) 
			WHERE post_expire <='$time' AND post_type = 'ims_order' AND post_status = 'draft'"
		);
		
		do_action( 'ims_after_cron' );
	}
		
	/**
	*Rewrites for custom page managers
	*
	*@param array $wp_rewrite
	*@return array
	*@since 0.5.0 
	*/
	 function add_rewrite_rules( $wp_rewrite ){
		
		if( empty( $this->pages ) )
			$this->load_pages( );
		
		$wp_rewrite->add_rewrite_tag( "%gallery%", '([^/]+)', "ims_gallery=" );
		$wp_rewrite->add_rewrite_tag( '%paypalipn%', '([^/]+)', 'paypalipn=' );
		$wp_rewrite->add_rewrite_tag( '%imslogout%', '([^/]+)', 'imslogout=' );
		$wp_rewrite->add_rewrite_tag( '%imsmessage%', '([0-9]+)', 'imsmessage=' );
		$wp_rewrite->add_permastruct( 'ims_gallery', __( 'galleries', $this->domain ) . '/%ims_gallery%/%imspage%', false );
		$galleries = preg_match( '/[^\\p{Common}\\p{Latin}]/u', __( 'galleries', $this->domain ) ) ? 'galleries' : __( 'galleries', $this->domain );
	
		$new_rules[$galleries . "/([^/]+)/feed/(imstore)/?$"] = 
		"index.php?ims_gallery=" . $wp_rewrite->preg_index(1)."&feed=" . $wp_rewrite->preg_index(2 );
		
		$new_rules[$galleries . "/([^/]+)/logout/?$"] = "index.php?ims_gallery=".$wp_rewrite->preg_index(1) . '&imslogout=1';
		
		foreach( $this->pages as $id => $page ){
			$slug = ( preg_match( '/[^\\p{Common}\\p{Latin}]/u', $page )) ? $id : sanitize_title($page );
			if( $id == 'photos' ){
				$new_rules[$galleries . "/([^/]+)/page/?([0-9]+)/?$"] = 
				"index.php?ims_gallery=". $wp_rewrite->preg_index(1). "&imspage=$id".
				'&paged='.$wp_rewrite->preg_index(2);
				
				$new_rules[$galleries . "/([^/]+)/$slug/page/?([0-9]+)/?$"] = 
				"index.php?ims_gallery=". $wp_rewrite->preg_index(1). "&imspage=$id".
				'&paged='.$wp_rewrite->preg_index(2);
				
				$new_rules[$galleries . "/([^/]+)/$slug/page/?([0-9]+)/ms/?([0-9]+)/?$"] = 
				"index.php?ims_gallery=". $wp_rewrite->preg_index(1). "&imspage=$id".
				'&paged='.$wp_rewrite->preg_index(2).'&imsmessage='.$wp_rewrite->preg_index(3);
			}
					
			$new_rules[$galleries . "/([^/]+)/$slug/ms/?([0-9]+)/?$"] = 
			"index.php?ims_gallery=".$wp_rewrite->preg_index(1). "&imspage=$id".
			'&imsmessage='.$wp_rewrite->preg_index(2);
		
			$new_rules[$galleries . "/([^/]+)/$slug/?$"] = 
			"index.php?ims_gallery=" . $wp_rewrite->preg_index(1)."&imspage=$id";
		}
			
		$wp_rewrite->rules = $new_rules + $wp_rewrite->rules;
		$wp_rewrite->rules["/page/?([0-9]+)/?$"] = "index.php?paged=" . $wp_rewrite->preg_index(1 ); //print_r($wp_rewrite );

		return $wp_rewrite;
	}

	/**
	*Define contant variables
	*
	*@return void
	*@since 0.5.0 
	*/
	function define_constant( ){
		do_action( 'ims_define_constants', IMSTORE_ABSPATH );
		
		define( 'IMSTORE_URL', WP_PLUGIN_URL . "/" . IMSTORE_FOLDER );
		define( 'IMSTORE_ADMIN_URL', IMSTORE_URL . '/admin' );
		
		if( !defined( 'WP_SITE_URL' ) ) define( 'WP_SITE_URL', get_bloginfo( 'url' ) );
		if( !defined( 'WP_CONTENT_URL' ) ) define( 'WP_CONTENT_URL', get_bloginfo( 'wpurl') . '/wp-content' );
		if( !defined( 'WP_TEMPLATE_DIR' ) ) define( 'WP_TEMPLATE_DIR',get_template_directory( ) );

		$this->key = apply_filters( 'ims_image_key', substr( preg_replace( "([^a-zA-Z0-9])", '', NONCE_KEY ),0,15 ) );
		
		if( !file_exists( IMSTORE_ABSPATH."/admin/_key/{$this->key}.txt" ) ){
			$fh = @fopen(IMSTORE_ABSPATH."/admin/_key/{$this->key}.txt", 'w');  
			@fclose($fh); @chmod(IMSTORE_ABSPATH."/admin/_key/{$this->key}.txt", 0640 );
		}
	}
		
	/**
	*Register localization/language file
	*
	*@return void
	*@since 0.5.0 
	*/
	function load_text_domain( ){
		$locale 	= get_locale( );
		$filedir 	= $this->content_dir . '/languages/_ims/'. $this->domain . '-' . $locale . '.mo';
		
		if( function_exists( 'load_plugin_textdomain' ) )
			load_plugin_textdomain( $this->domain, false, apply_filters( 'ims_load_textdomain', '../languages/_ims/', $this->domain , $locale ) );
		elseif( function_exists( 'load_textdomain' ) )
			load_textdomain( $this->domain, apply_filters( 'ims_load_textdomain', $filedir, $this->domain , $locale ) );
	}
	
	/**
	*Get all packages
	*
	*@return array
	*@since 3.0.0
	*/
	function get_packages( ){
		global $wpdb;
		return $wpdb->get_results("SELECT DISTINCT ID,post_title FROM $wpdb->posts WHERE post_type = 'ims_package'" );
	}
	
	/**
	*load pages to use for permalinks
	*and to display the correct section
	*
	*@return void
	*@since 0.5.0 
	*/
	function load_pages( ){
		$this->pages['photos'] 			= __( 'Photos', $this->domain );
		$this->pages['slideshow'] 		= __( 'Slideshow', $this->domain );
		$this->pages['favorites'] 		= __( 'Favorites', $this->domain );

		if( empty( $this->opts['disablestore'] ) ){
			$this->pages['price-list'] 		= __( 'Price List', $this->domain );
			$this->pages['shopping-cart']	= __( 'Shopping Cart', $this->domain );
			$this->pages['receipt']			= __( 'Receipt', $this->domain );
			$this->pages['checkout'] 		= __( 'Checkout', $this->domain );
		}
		$this->pages = apply_filters( 'ims_load_pages', $this->pages );
	}
	
	/**
	*load color options
	*
	*@return void
	*@since 0.5.0 
	*/
	function load_color_opts( ){
		$this->color = array( 
			'ims_color' => __( 'Full Color', $this->domain ),
			'ims_sepia' => __( 'Sepia ', $this->domain ),
			'ims_bw' => __( 'B &amp; W', $this->domain ),
		 );
		 	
		if( isset($this->opts['disablebw'] ) )
			unset( $this->color['ims_bw'] );
			
		if( isset($this->opts['disablesepia'] ) )
			unset( $this->color['ims_sepia'] );
			
		$this->color = apply_filters( 'ims_color_opts', $this->color );
	}
	
	/**
	*Add user capabilities to current user
	*
	*@return void
	*@since 3.0.0 
	*/
	function set_user_caps( ){
		global $current_user;
		
		if ( !isset($current_user->ID) || 
		isset($current_user->caps['administrator']) )
			return;

		if( !empty( $current_user->ims_user_caps ) )
			$current_user->allcaps += $current_user->ims_user_caps;
	}
	
	/**
	*Register custom post types
	*
	*@return void
	*@since 3.0.0
	*/
	function register_post_types( ){
		
		//register image type to be able to edit images
		register_post_type( 'ims_image',
			array( 
				'public' => false,
				'show_ui' => false,
				'revisions' => false,
				'publicly_queryable' => true,
				'exclude_from_search' => true,
				'show_in_nav_menus' => false,
			 )
		 ); 
		
		$searchable = ( get_option( 'ims_searchable' ) ) ? false : true;
		$texedit = isset( $this->opts['showtexteditor'] ) ? 'editor' : false;
		
		//register gallery post type assign ims_album taxonomy
		$posttype = apply_filters( 'ims_gallery_post_type', array( 
			'labels' => array( 
				'name' 				=> _x( 'Galleries', 'post type general name', $this->domain ),
				'singular_name' => _x( 'Gallery', 'post type singular name', $this->domain ),
				'add_new' 			=> _x( 'Add New', 'Gallery', $this->domain ),
				'add_new_item'	=> __( 'Add New Gallery', $this->domain ),
				'edit_item' 		=> __( 'Edit Gallery', $this->domain ),
				'new_item' 		=> __( 'New Gallery', $this->domain ),
				'view_item' 		=> __( 'View Gallery', $this->domain ),
				'search_items' 	=> __( 'Search galleries', $this->domain ),
				'not_found' 		=> __( 'No galleries found', $this->domain ),
				'not_found_in_trash' => __( 'No galleries found in Trash', $this->domain ),
			 ),
			'public' 				=> true,
			'show_ui' 			=> true,
			'menu_position' 	=> 33,
			'publicly_queryable'=> true,
			'hierarchical' 		=> false,
			'revisions'			=> false,
			'capability_type' 	=> 'page',
			'query_var'			=> 'ims_gallery',
			'show_in_nav_menus' => false,
			'exclude_from_search'=> $searchable,
			'menu_icon' 		=> IMSTORE_URL.'/_img/imstore.png',
			'supports' 			=> array( 'title', 'comments', 'author', 'excerpt', 'page-attributes', $texedit ), 
			'rewrite' 			=> array( 'slug' => __( 'galleries', $this->domain ), 'with_front'=>false ),
			'taxonomies'		=> array( 'ims_album' )
		 ));
		
		register_post_type( 'ims_gallery', $posttype );

		$statuses = array(
				'expire' 	=> __( 'Expired', $this->domain ),
				'closed' 	=> __( 'Closed', $this->domain ),
				'shipped' => __( 'Shipped', $this->domain ),
				'cancelled' => __( 'Cancelled', $this->domain ),
		);
		
		foreach( $statuses as $status => $label ){
			register_post_status( $status, array(
				'protected' 	=> true, 
				'publicly_queryable' => false, 'label' => $label,
				'label_count' 	=> _n_noop("{$label} <span class='count'>(%s)</span>","{$label} <span class='count'>(%s)</span>")
			));
		}
		
		//register taxomomy albums
		register_taxonomy( 'ims_album', array( 'ims_gallery' ), array( 
			'labels' => array( 
				'name' 			=> _x( 'Albums', 'taxonomy general name', $this->domain ),
				'singular_name' => _x( 'Album', 'taxonomy singular name', $this->domain ),
				'search_items' 	=> __( 'Search Albums', $this->domain ),
				'all_items' 			=> __( 'All Albums', $this->domain ),
				'parent_item' 		=> __( 'Parent Album', $this->domain ),
				'parent_item_colon' => __( 'Parent Album:', $this->domain ),
				'edit_item' 		=> __( 'Edit Album', $this->domain ), 
				'update_item' 	=> __( 'Update Album', $this->domain ),
				'add_new_item' 	=> __( 'Add New Album', $this->domain ),
				'new_item_name' => __( 'New Album Name', $this->domain ),
				'menu_name' 	=> __( 'Album', $this->domain ),
			 ),
			'show_ui' 		=> true,
			'query_var' 	=> true,
			'hierarchical' 	=> true,
			'show_in_nav_menus' => true,
			'rewrite' 		=> array( 'slug' =>__( 'albums', $this->domain ) ),
		 ) );
		
	}
		
	/**
	*Fast in_array function
	*
	*@parm string $elem
	*@parm array $array
	*@return bool
	*@since 3.0.0
	*/
	function in_array( $elem, $array ){ 
		foreach( $array as $val )
			if( $val == $elem ) return true; 
		return false; 
	}
		
	/**
	*Encrypt image ID for downlaods
	*
	*@param $id unit
	*@return string
	*@since 2.0.1
	*/
	function encrypt_id( $id ){
		return substr(md5($id ), 0, 8).dechex($id );
	}
	
	/**
	*Dencrypt image ID for downlaods
	*
	*@param $id string
	*@return unit
	*@since 2.0.1
	*/
	function decrypt_id($id){
		$md5_8 = substr($id, 0, 8 );
		$real_id = hexdec(substr($id, 8) );
		return ($md5_8==substr(md5($real_id ), 0, 8)) ? $real_id : 0;
 }
	
	/**
	 *Format price
	 *
	 *@parm unit $price
	 *@parm string $before
	 *@parm string $after
	 *@return string
	 *@since 3.0.0
	 */	
	function format_price( $price , $before='', $after = '' ){
		if ( !is_numeric( $price ))  $price = 0 ;
		
		if( empty( $this->opts['disable_decimal']  ) ) 
			$price = number_format( $price, 2 );
			
		return sprintf( $before . $this->cformat[$this->loc], $price . $after );
	}
		
	/**
	 * Error messages
	 *
	 *@param obj $errors
	 *@param bol $retrun
	 *@return string|null
	 *@since 3.0.0
	 */	
	function error_message( $errors, $return = false ){
		$error = '<div class="ims-message ims-error error">' . "\n";
		foreach( $errors->get_error_messages( )as $err )
			$error .= "<p><strong>$err</strong></p>\n";
		$error .= '</div>' . "\n"; 
		
		if( $return ) return $error;
		else echo $error;
	}
}

/**
* Escaping for textarea values.
*
* @param string $text
* @return string
* @since 3.0.0
*/
if( !function_exists( 'esc_textarea') ){
	function esc_textarea( $text ){
	$safe_text = htmlspecialchars( $text, ENT_QUOTES );
 		return apply_filters( 'esc_textarea', $safe_text, $text );
	}
}
?>